/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// ===================================================================
/*! @class Biff
    @brief Main class of gnubiff
    
    The Biff class is the main class of gnubiff since it allows other
    class to access parameters. This class is also responsible for
    loading and saving a configuration file in XML format.
    There can be only one Biff object throughout application.
*/
// ===================================================================

#ifndef _BIFF_H
#define _BIFF_H

#ifdef HAVE_CONFIG_H
#   include "../config/config.h"
#endif
#include <iostream>
#include <fstream>
#include <string>
#include <glib.h>
#include "Setup.h"
#include "Applet.h"

#ifdef ENABLE_NLS
#  include<libintl.h>
#  ifndef USE_GNOME
#    define _(String) dgettext(GETTEXT_PACKAGE,String)
#  else
#    include <gnome.h>
#  endif
#  ifdef gettext_noop
#    define N_(String) gettext_noop(String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define _(String) (String)
#  define N_(String) (String)
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,String) (String)
#  define dcgettext(Domain,String,Type) (String)
#  define bindtextdomain(Domain,Directory) (Domain)
#endif


// ===================================================================
// - Constant definitions --------------------------------------------
// ===================================================================
#define GTK_MODE		0
#define GNOME_MODE		1

#define PROTOCOL_FILE		0
#define PROTOCOL_POP3		1
#define PROTOCOL_IMAP4		2
#define PROTOCOL_MAILDIR	3
#define PROTOCOL_MH		4
#define PROTOCOL_APOP		5
#define DEFAULT_POP3_PORT	110
#define DEFAULT_IMAP4_PORT	143
#define DEFAULT_SSL_POP3_PORT	995
#define DEFAULT_SSL_IMAP4_PORT	993

#define	SOUND_NONE		0
#define SOUND_BEEP		1
#define SOUND_FILE		2

#define SIGN_PLUS		TRUE
#define SIGN_MINUS		FALSE

#define STATE_RUNNING		TRUE
#define STATE_STOPPED		FALSE

// ===================================================================
// - Type definitions ------------------------------------------------
// ===================================================================
#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef struct _header {
  string	sender;
  string	subject;
  string	date;
  string	body;
  string	charset;
  gint		status;
} header;
#endif

using namespace std;
class Biff {
  friend class Setup;
  // ===================================================================
  // - Public methods --------------------------------------------------
  // ===================================================================
public:
  //! @name Base functions
  Biff (gint mode=GTK_MODE, string filename = "");
  ~Biff (void);
  //@}

  //! @name I/O functions
  gboolean load (void);
  gboolean save (void);
  void xml_start_element (GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values, GError **error);
  void xml_error (GMarkupParseContext *context, GError *error);
  //@}

#ifndef DOXYGEN_SHOULD_SKIP_THIS
  //! @name Features consultations / modifications
  class Setup *setup (void)				{return _setup;}
  class Applet *applet (void)				{return _applet;}
  class Popup *popup (void)				{return _popup;}
  class Mailbox *mailbox (void)				{return _mailbox;}
  //@}
#endif


  // ===================================================================
  // - Public attributes -----------------------------------------------
  // ===================================================================
public:
  //! @name Mailbox
  guint		_mode;			/*!< \brief GTK or GNOME mode */
  //@}

  //! @name Mailbox
  string	_title;			/*!< \brief Title to be displayed in popup and applet */
  guint		_protocol;		/*!< \brief Protocol to be used */
  guint		_port;			/*!< \brief Port to be used with apop, pop3 or imap protocols */
  gboolean	_ssl;			/*!< \brief Use of SSL or not for apop, pop3 or imap4 protocols */
  string	_address;		/*!< \brief Address of the mailbox (file or hostname) */
  string	_user;			/*!< \brief User name for apop, pop3 or imap4 protocols */
  string	_password;		/*!< \brief Password for apop, pop3 or imap4 protocols */
  string	_certificate;		/*!< \brief Filename of the certificate or an empty string, if none should be used */
  //@}

  //! @name Options
  guint		_polltime;		/*!< \brief Time (seconds) between polls */
  guint		_poptime;		/*!< \brief Time (seconds) to display popup */
  guint		_sound_type;		/*!< \brief Type of sound to be played when new mail */
  string	_sound_file;		/*!< \brief Sound file to be used */
  guint		_sound_volume;		/*!< \brief Sound volume (0..100) */
  gboolean	_state;			/*!< \brief State of gnubiff (stopped or running) */
  //@}

  //! @name Advanced
  gboolean	_display_popup;		/*!< \brief Display popup (or not) when new mail */
  string	_mail_app;		/*!< \brief Filename of the mail client */
  //@}

  //! @name Layout
  string	_mail_image;		/*!< \brief Filename of the image or animation for new mail */
  string	_nomail_image;		/*!< \brief Filename of the image or animation for no new mail */
  gboolean	_hide_nomail;		/*!< \brief Hide (or not) biff or applet when no new mail */
  gboolean	_display_title;		/*!< \brief Display title (or not) */
  gboolean	_display_date;		/*!< \brief Display date (or not) in popup */
  guint		_max_sender_size;	/*!< \brief Maximum size (characters) of sender field in popup */
  guint		_max_subject_size;	/*!< \brief Maximum size (characters) of subject in popup */
  //@}

  //! @name Geometry
  gboolean	_popup_x_sign;		/*!< \brief Sign of x position of popup window */
  guint		_popup_x;		/*!< \brief X position of popup window */
  gboolean	_popup_y_sign;		/*!< \brief Sign of y position of popup window */
  guint		_popup_y;		/*!< \brief Y position of popup window */
  gboolean	_popup_decorated;	/*!< \brief Decorations for popup window (or not) */
  gboolean	_popup_no_geometry;	/*!< \brief Use geometry (or not) */
  gboolean	_biff_x_sign;		/*!< \brief Sign of x position of biff window */
  guint		_biff_x;		/*!< \brief X position of biff window */
  gboolean	_biff_y_sign;		/*!< \brief Sign of y position of biff window */
  guint		_biff_y;		/*!< \brief Y position of biff window */
  gboolean	_biff_decorated;	/*!< \brief Decorations for biff window (or not) */
  gboolean	_biff_no_geometry;	/*!< \brief Use geometry (or not) */
  guint		_max_header_display;	/*!< \brief Maximum number of headers to be collected and displayed */
  //@}

  //! @name Font
  string	_popup_title_font;	/*!< \brief Popup body font */
  string	_popup_body_font;	/*!< \brief Popup title font */
  string	_biff_title_font;	/*!< \brief Biff body font */
  string	_biff_body_font;	/*!< \brief Biff title font */
  string	_applet_title_font;	/*!< \brief Applet title font */
  string	_applet_body_font;	/*!< \brief Applet body font */
  string	_popup_title_color;	/*!< \brief Popup title font color */
  string	_popup_body_color;	/*!< \brief Popup bbody font color */
  string	_biff_title_color;	/*!< \brief Biff title font color */
  string	_biff_body_color;	/*!< \brief Biff body font color */
  string	_applet_title_color;	/*!< \brief Applet title font color */
  string	_applet_body_color;	/*!< \brief Applet body font color */
  //@}


  // ===================================================================
  // - Protected methods -----------------------------------------------
  // ===================================================================
protected:

  // ===================================================================
  // - Protected attributes --------------------------------------------
  // ===================================================================
protected:



  // ===================================================================
  // - Private methods -------------------------------------------------
  // ===================================================================
private:

  // ===================================================================
  // - Private attributes ----------------------------------------------
  // ===================================================================
private:
  string		_filename;	/*!< \brief Configuration filename */
  class Setup *		_setup;		/*!< \brief Setup window (GUI) */
  class Applet *	_applet;	/*!< \brief Applet (GUI) */
  class Popup *		_popup;		/*!< \brief Popup window (GUI) */
  class Mailbox *	_mailbox;	/*!< \brief Mailbox */
  string		_table;		/*!< \brief Table used for crypting password */
};

#endif /* _BIFF_H */
